@@ -1,32 +1,30 @@
-//----------------------------------------------------------------------------
-//  Copyright (C) 2012  The IPython Development Team
-//
-//  Distributed under the terms of the BSD License.  The full license is in
-//  the file COPYING, distributed as part of this software.
-//----------------------------------------------------------------------------
+// Copyright (c) IPython Development Team.
+// Distributed under the terms of the Modified BSD License.
 
-//============================================================================
-// CellToolbar
-//============================================================================
-
-
-/**
- * A Module to control the per-cell toolbar.
- * @module IPython
- * @namespace IPython
- * @submodule CellToolbar
- */
-var IPython = (function (IPython) {
+define([
+    'base/js/namespace',
+    'jquery',
+    'base/js/events'
+], function(IPython, $, events) {
     "use strict";
 
-    /**
-     * @constructor
-     * @class CellToolbar
-     * @param {The cell to attach the metadata UI to} cell
-     */
-    var CellToolbar = function (cell) {
+    var CellToolbar = function (options) {
+        /**
+         * Constructor
+         *
+         * Parameters:
+         *  options: dictionary
+         *      Dictionary of keyword arguments.
+         *          events: $(Events) instance 
+         *          cell: Cell instance
+         *          notebook: Notebook instance 
+         *
+         *  TODO: This leaks, when cell are deleted
+         *  There is still a reference to each celltoolbars.
+         */
         CellToolbar._instances.push(this);
-        this.cell = cell;
+        this.notebook = options.notebook;
+        this.cell = options.cell;
         this.create_element();
         this.rebuild();
         return this;
@@ -34,10 +32,9 @@ var IPython = (function (IPython) {
 
 
     CellToolbar.prototype.create_element = function () {
-        this.inner_element = $('<div/>').addClass('celltoolbar')
+        this.inner_element = $('<div/>').addClass('celltoolbar');
         this.element = $('<div/>').addClass('ctb_hideshow')
             .append(this.inner_element);
-        this.show();
     };
 
 
@@ -118,7 +115,9 @@ var IPython = (function (IPython) {
      * @method register_callback
      * @param name {String} name to use to refer to the callback. It is advised to use a prefix with the name
      * for easier sorting and avoid collision
-     * @param  callback {function(div, cell)} callback that will be called to generate the ui element
+     * @param callback {function(div, cell)} callback that will be called to generate the ui element
+     * @param [cell_types] {List_of_String|undefined} optional list of cell types. If present the UI element
+     * will be added only to cells of types in the list.
      *
      *
      * The callback will receive the following element :
@@ -154,9 +153,9 @@ var IPython = (function (IPython) {
      *      // user the ability to use it later
      *      CellToolbar.register_callback('foo', toggle);
      */
-    CellToolbar.register_callback = function(name, callback){
+    CellToolbar.register_callback = function(name, callback, cell_types) {
         // Overwrite if it already exists.
-        CellToolbar._callback_dict[name] = callback;
+        CellToolbar._callback_dict[name] = cell_types ? {callback: callback, cell_types: cell_types} : callback;
     };
 
 
@@ -166,7 +165,7 @@ var IPython = (function (IPython) {
      * @method register_preset
      * @param name {String} name to use to refer to the preset. It is advised to use a prefix with the name
      * for easier sorting and avoid collision
-     * @param  preset_list {List of String} reverse order of the button in the toolbar. Each String of the list
+     * @param  preset_list {List_of_String} reverse order of the button in the toolbar. Each String of the list
      *          should correspond to a name of a registerd callback.
      *
      * @private
@@ -181,15 +180,32 @@ var IPython = (function (IPython) {
      *      CellToolbar.register_preset('foo.foo_preset1', ['foo.c1', 'foo.c2', 'foo.c5'])
      *      CellToolbar.register_preset('foo.foo_preset2', ['foo.c4', 'foo.c5'])
      */
-    CellToolbar.register_preset = function(name, preset_list) {
+    CellToolbar.register_preset = function(name, preset_list, notebook) {
         CellToolbar._presets[name] = preset_list;
-        $([IPython.events]).trigger('preset_added.CellToolbar', {name: name});
+        events.trigger('preset_added.CellToolbar', {name: name});
         // When "register_callback" is called by a custom extension, it may be executed after notebook is loaded.
         // In that case, activate the preset if needed.
-        if (IPython.notebook && IPython.notebook.metadata && IPython.notebook.metadata.celltoolbar === name)
-            this.activate_preset(name);
+        if (notebook && notebook.metadata && notebook.metadata.celltoolbar === name){
+            CellToolbar.activate_preset(name);
+        }
     };
 
+    /**
+     * unregister the selected preset, 
+     *
+     * return true if preset successfully unregistered
+     * false otherwise
+     *
+     **/
+    CellToolbar.unregister_preset = function(name){
+        if(CellToolbar._presets[name]){
+            delete CellToolbar._presets[name];
+            events.trigger('unregistered_preset.CellToolbar', {name: name});
+            return true
+        }
+        return false
+    }
+
 
     /**
      * List the names of the presets that are currently registered.
@@ -228,7 +244,7 @@ var IPython = (function (IPython) {
             CellToolbar.rebuild_all();
         }
 
-        $([IPython.events]).trigger('preset_activated.CellToolbar', {name: preset_name});
+        events.trigger('preset_activated.CellToolbar', {name: preset_name});
     };
 
 
@@ -250,11 +266,13 @@ var IPython = (function (IPython) {
      * @method rebuild
      */
     CellToolbar.prototype.rebuild = function(){
-        // strip evrything from the div
-        // which is probably inner_element
-        // or this.element.
+        /**
+         * strip evrything from the div
+         * which is probably inner_element
+         * or this.element.
+         */
         this.inner_element.empty();
-        this.show();
+        this.ui_controls_list = [];
 
         var callbacks = CellToolbar._callback_dict;
         var preset = CellToolbar._ui_controls_list;
@@ -263,10 +281,16 @@ var IPython = (function (IPython) {
             var key = preset[i];
             var callback = callbacks[key];
             if (!callback) continue;
+
+            if (typeof callback === 'object') {
+                if (callback.cell_types.indexOf(this.cell.cell_type) === -1) continue;
+                callback = callback.callback;
+            }
             
             var local_div = $('<div/>').addClass('button_container');
             try {
                 callback(local_div, this.cell, this);
+                this.ui_controls_list.push(key);
             } catch (e) {
                 console.log("Error in cell toolbar callback " + key, e);
                 continue;
@@ -274,11 +298,16 @@ var IPython = (function (IPython) {
             // only append if callback succeeded.
             this.inner_element.append(local_div);
         }
+
+        // If there are no controls or the cell is a rendered TextCell hide the toolbar.
+        if (!this.ui_controls_list.length) {
+            this.hide();
+        } else {
+            this.show();
+        }
     };
 
 
-    /**
-     */
     CellToolbar.utils = {};
 
 
@@ -333,17 +362,48 @@ var IPython = (function (IPython) {
                         setter(cell, !v);
                         chkb.attr("checked", !v);
             });
-            button_container.append($('<div/>').append(lbl));
+            button_container.append($('<span/>').append(lbl));
         };
     };
 
 
+    /**
+     * A utility function to generate bindings between a input field and cell/metadata
+     * @method utils.input_ui_generator
+     * @static
+     *
+     * @param name {string} Label in front of the input field
+     * @param setter {function( cell, newValue )}
+     *        A setter method to set the newValue
+     * @param getter {function( cell )}
+     *        A getter methods which return the current value.
+     *
+     * @return callback {function( div, cell )} Callback to be passed to `register_callback`
+     *
+     */
+    CellToolbar.utils.input_ui_generator = function(name, setter, getter){
+        return function(div, cell, celltoolbar) {
+            var button_container = $(div);
+
+            var text = $('<input/>').attr('type', 'text');
+            var lbl = $('<label/>').append($('<span/>').text(name));
+            lbl.append(text);
+            text.attr("value", getter(cell));
+
+            text.keyup(function(){
+                setter(cell, text.val());
+            });
+            button_container.append($('<span/>').append(lbl));
+            IPython.keyboard_manager.register_events(text);
+        };
+    };
+
     /**
      * A utility function to generate bindings between a dropdown list cell
      * @method utils.select_ui_generator
      * @static
      *
-     * @param list_list {list of sublist} List of sublist of metadata value and name in the dropdown list.
+     * @param list_list {list_of_sublist} List of sublist of metadata value and name in the dropdown list.
      *        subslit shoud contain 2 element each, first a string that woul be displayed in the dropdown list,
      *        and second the corresponding value to  be passed to setter/return by getter. the corresponding value 
      *        should not be "undefined" or behavior can be unexpected.
@@ -381,12 +441,12 @@ var IPython = (function (IPython) {
      *      CellToolbar.register_callback('slideshow.select', select_type);
      *
      */
-    CellToolbar.utils.select_ui_generator = function(list_list, setter, getter, label, cell_types){
+    CellToolbar.utils.select_ui_generator = function(list_list, setter, getter, label) {
         label = label || "";
         return function(div, cell, celltoolbar) {
             var button_container = $(div);
             var lbl = $("<label/>").append($('<span/>').text(label));
-            var select = $('<select/>').addClass('ui-widget ui-widget-content');
+            var select = $('<select/>');
             for(var i=0; i < list_list.length; i++){
                 var opt = $('<option/>')
                     .attr('value', list_list[i][1])
@@ -397,18 +457,12 @@ var IPython = (function (IPython) {
             select.change(function(){
                         setter(cell, select.val());
                     });
-            button_container.append($('<div/>').append(lbl).append(select));
-            if (cell_types && cell_types.indexOf(cell.cell_type) == -1) {
-                celltoolbar.hide();
-            } else {
-                celltoolbar.show();
-            }
-
+            button_container.append($('<span/>').append(lbl).append(select));
         };
     };
 
-
+    // Backwards compatability.
     IPython.CellToolbar = CellToolbar;
 
-    return IPython;
-}(IPython));
+    return {'CellToolbar': CellToolbar};
+});
